<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Util;

use CleverReach\WooCommerce\Components\Repositories\Base_Repository;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Init_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use Exception;
use wpdb;

/**
 * Class Database
 *
 * @package CleverReach\WooCommerce\Components\Utility
 */
class Database {
	const BASE_TABLE = 'cleverreach_wc_entity';
	const OLD_CONFIG_TABLE = 'cleverreach_wc_config';

	/**
	 * WordPress database session.
	 *
	 * @var wpdb WordPress database
	 */
	private $db;

	/**
	 * Database constructor.
	 *
	 * @param wpdb $db Wodrpess database.
	 */
	public function __construct( $db ) {
		$this->db = $db;
	}

	/**
	 * Checks if plugin was already installed and initialized.
	 *
	 * @return bool
	 */
	public function plugin_already_initialized() {
		$table_name     = $this->db->prefix . self::BASE_TABLE;
		$old_table_name = $this->db->prefix . self::OLD_CONFIG_TABLE;

		return $this->db->get_var( "SHOW TABLES LIKE '" . $table_name . "'" ) === $table_name
		       || $this->db->get_var( "SHOW TABLES LIKE '" . $old_table_name . "'" ) === $old_table_name;
	}

	/**
	 * Checks if plugin version older than v3 was already installed and initialized.
	 *
	 * @return bool
	 */
	public function plugin_older_than_v3_already_initialized() {
		$old_table_name = $this->db->prefix . self::OLD_CONFIG_TABLE;

		return $this->db->get_var( "SHOW TABLES LIKE '" . $old_table_name . "'" ) === $old_table_name;
	}

	/**
	 * Executes installation scripts.
	 */
	public function install() {
		$query = $this->get_install_query();
		$this->db->query( $query );
	}

	/**
	 * Executes uninstallation.
	 */
	public function uninstall() {
		$query = 'DROP TABLE IF EXISTS ' . Base_Repository::get_table_name( self::BASE_TABLE );
		$this->db->query( $query );
	}

	/**
	 * Get config value form old table
	 *
	 * @param string $key key.
	 */
	public function get_old_config_value( $key ) {
		$query = 'SELECT `value` FROM ' . Base_Repository::get_table_name( self::OLD_CONFIG_TABLE ) .
		         " WHERE `key` = '$key'";

		return $this->db->get_var( $query );
	}

	/**
	 * Executes update database functions.
	 *
	 * @param Version_File_Reader $version_file_reader Version file reader.
	 *
	 * @return bool
	 */
	public function update( $version_file_reader ) {
		while ( $version_file_reader->has_next() ) {
			$statements = $version_file_reader->read_next();
			foreach ( $statements as $statement ) {
				try {
					$this->db->query( $statement );
				} catch ( Exception $ex ) {
					Init_Service::init();
					Logger::logInfo( $ex->getMessage(), 'Database Update' );
					Logger::logInfo( $statement, 'SQL' );

					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Prepares database queries for inserting tables.
	 *
	 * @return string
	 */
	private function get_install_query() {
		return 'CREATE TABLE IF NOT EXISTS `'
		       . Base_Repository::get_table_name( self::BASE_TABLE ) . '` (
            `id` INT(11) NOT NULL AUTO_INCREMENT,
            `type` VARCHAR(255) NOT NULL,
            `index_1` VARCHAR(255),
            `index_2` VARCHAR(255),
            `index_3` VARCHAR(255),
            `index_4` VARCHAR(255),
            `index_5` VARCHAR(255),
            `index_6` VARCHAR(255),
            `index_7` VARCHAR(255),
            `index_8` VARCHAR(255),
            `index_9` VARCHAR(255),
            `index_10` VARCHAR(255),
            `data` TEXT,
            PRIMARY KEY (`id`)
        ) DEFAULT CHARACTER SET utf8';
	}

	/**
	 * Truncates main table
	 */
	public function truncate_main_table() {
		$table_name = Base_Repository::get_table_name( self::BASE_TABLE );
		$query      = "TRUNCATE $table_name;";
		$this->db->query( $query );
	}
}
