<?php

namespace CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Tasks\Trigger;

use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Http\Proxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Interfaces\AutomationRecordService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Interfaces\Required\CartAutomationTriggerService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Tasks\Trigger\Filter\FilterChain;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Scheduler\Interfaces\Schedulable;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Serializer\Serializer;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\AbortTaskExecutionException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Task;

class TriggerCartAutomationTask extends Task implements Schedulable
{
    /**
     * @var int
     */
    protected $recordId;

    /**
     * Transforms serializable object into an array.
     *
     * @return array Array representation of a serializable object.
     */
    public function toArray()
    {
        return array('recordId' => $this->recordId);
    }

    /**
     * Transforms array into an serializable object,
     *
     * @param array $array Data that is used to instantiate serializable object.
     *
     * @return \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Serializer\Interfaces\Serializable
     *      Instance of serialized object.
     */
    public static function fromArray(array $array)
    {
        return new static($array['recordId']);
    }

    /**
     * String representation of object
     *
     * @link https://php.net/manual/en/serializable.serialize.php
     *
     * @return string the string representation of the object or null
     */
    public function serialize()
    {
        return Serializer::serialize($this->recordId);
    }

    /**
     * Constructs the object
     *
     * @param string $serialized
     *
     * @return void
     */
    public function unserialize($serialized)
    {
        $this->recordId = Serializer::unserialize($serialized);
    }

    /**
     * TriggerCartAutomationTask constructor.
     *
     * @param int $recordId
     */
    public function __construct($recordId)
    {
        $this->recordId = $recordId;
    }

    /**
     * Triggers abandoned cart automation.
     *
     * @throws \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Authorization\Exceptions\FailedToRefreshAccessToken
     * @throws \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Authorization\Exceptions\FailedToRetrieveAuthInfoException
     * @throws \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Exceptions\FailedToDeleteAutomationRecordException
     * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Http\Exceptions\HttpCommunicationException
     * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Http\Exceptions\HttpRequestException
     * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\AbortTaskExecutionException
     */
    public function execute()
    {
        $record = $this->getRecordService()->find($this->recordId);
        if ($record === null) {
            throw new AbortTaskExecutionException('Record not found.');
        }

        $this->reportProgress(30);

        $trigger = $this->getCartAutomationTriggerService()->getTrigger($record->getCartId());
        if ($trigger === null) {
            $this->getRecordService()->delete($this->recordId);

            throw new AbortTaskExecutionException('Trigger is not found.');
        }

        $this->reportProgress(60);

        try {
            FilterChain::execute($record, $trigger);
        } catch (\Exception $e) {
            $this->getRecordService()->delete($this->recordId);

            throw new AbortTaskExecutionException('Trigger failed to pass filter.', $e->getCode(), $e);
        }

        $this->reportProgress(90);

        $this->getProxy()->trigger($trigger);

        $this->reportProgress(95);

        $this->getRecordService()->delete($this->recordId);

        $this->reportProgress(100);
    }

    /**
     * Provides automation record service.
     *
     * @return AutomationRecordService | object
     */
    private function getRecordService()
    {
        return ServiceRegister::getService(AutomationRecordService::CLASS_NAME);
    }

    /**
     * Provides automation trigger service.
     *
     * @return CartAutomationTriggerService | object
     */
    private function getCartAutomationTriggerService()
    {
        return ServiceRegister::getService(CartAutomationTriggerService::CLASS_NAME);
    }

    /**
     * Provides proxy class.
     *
     * @return Proxy | object
     */
    private function getProxy()
    {
        return ServiceRegister::getService(Proxy::CLASS_NAME);
    }

    /**
     * Defines whether schedulable task can be enqueued for execution if there is already instance with queued status.
     *
     * @return bool False indicates that the schedulable task should not enqueued if there
     *      is already instance in queued status.
     */
    public function canHaveMultipleQueuedInstances()
    {
        return true;
    }
}